       integer   maxshells, p_number
       real*8    max_p, mp,vo, p0 
c
       parameter (maxshells = 200, max_p = 10.0D0, p_number = 250, 
     *      p0 = 2.0, vo = 52483.25D0, mp = 1.67D-27) 
c     
