       subroutine bndyall
c
c    dac:zeus3d.bndyall <------------ sets all hydrodynamical boundaries
c                                                            march, 1990
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE:  This routine collects the calls to the subroutines BVAL*
c  which update the boundaries of the hydrodynamical variables.  The
c  boundaries of the magnetic fields are intentionally omitted, since
c  they are computed directly in the CT algorithm.
c
c  INPUT VARIABLES: [NONE]
c
c  OUTPUT VARIABLES: [NONE]
c
c  LOCAL VARIABLES: [NONE]
c
c  EXTERNALS:
c    BVALD
c    BVALE
c    BVALV1  , BVALV2, BVALV3
c
c-----------------------------------------------------------------------
c
c
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
c
c      External statements
c
       external      bvald   , bvale   , bvalv1  , bvalv2  , bvalv3
c
c-----------------------------------------------------------------------
c
       call bvald
       call bvale
       call bvalv1
       call bvalv2
       call bvalv3
c
       return
       end
