       subroutine findno ( msg, istrt, ifin )
c
c    jms:zeus3d.findno <-------- finds reset number in interrupt message
c                                                             june, 1988
c
c    written by: Jim Stone
c    modified 1: January, 1990 by David Clarke; incorporated into ZEUS3D
c
c  PURPOSE:  Finds beginning and ending of a reset number in an
c  interrupt message string.
c
c  INPUT VARIABLES:
c    msg      character*80 string containing the message
c
c  OUTPUT VARIABLES:
c    istrt    number of first character of reset number in message
c             If no number is found, a value of -1 is returned
c    ifin     number of last character of reset number in message
c
c  LOCAL VARIABLES:
c
c-----------------------------------------------------------------------
c
c
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*80  msg
       integer       istrt   , ifin    , iblnk
c
c-----------------------------------------------------------------------
c
       istrt = 0
       ifin  = 0
c
       do 10 iblnk=1,80
         if (msg(iblnk:iblnk) .eq. ' ') go to 20
10     continue
20     continue
       do 30 istrt=iblnk,80
         if (msg(istrt:istrt) .ne. ' ') go to 40
30     continue
       istrt = -1
       return
c
40     continue
       do 50 ifin=istrt,80
         if (msg(ifin:ifin) .eq.  ' ') go to 60
50     continue
60     continue
       ifin = ifin - 1
c
       return
       end
