       subroutine trnsps4
     1   ( ar, idim, jdim, art )
c
c    dac:zeus3d.trnsps4 <------------------------ transposes a 2-D array
c                                                          october, 1992
c
c    written by: David Clarke
c    modified 1: Robert Fiedler to make the input and output arrays
c                real*4.  Called only if alias REAL is set to real*4.
c
c  PURPOSE:  This subroutine returns the transpose of a 2-D array.
c
c  INPUT VARIABLES:
c    ar          input array to be transposed
c    idim,jdim   dimensions of "ar" and "art"
c
c  OUTPUT VARIABLES:
c    art         transposed array
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [NONE]
c
c-----------------------------------------------------------------------
c
c
c-----------------------------------------------------------------------
c------------------------ IMPLICIT STATEMENTS --------------------------
c-----------------------------------------------------------------------
c
       implicit      none
       integer       idim    , jdim    , i       , j
c
       real*4        ar      (idim,jdim), art     (jdim,idim)
c
c-----------------------------------------------------------------------
c
       do 20 i=1,idim
         do 10 j=1,jdim
           art(j,i) = ar(i,j)
10       continue
20     continue
c
       return
       end
